% Units: the units you put in for wavelength and layer thicknesses
% determnins the length units. 
% The output will be in length^-1, which is
% the ratio of (X/volume) / (X/area) where X is watts or joules and your
% laser is specified in X/area. X/volume will be in units of what you use
% for "X per area" and what you use for lambda and depth. 

% Normal use will be to have your laser in J or W per cm2 and the wavelength and
% thicknesses in nm. In this case, the units for the data calculated by
% this program will be nm^-1, and if you multiply that by J/cm2 or W/cm2,
% you will get (W or J) nm-1 cm-2.  To convert to (J or W)/cm3 you would
% multiply by 10^7 nm/cm.  



clear all;
close all;

% %%%%%%%%%%%%%%%%%%%%%  Electrodeposited CuInSe2 on Mo on SLG example
% %%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%
% %  Excitation parameters.
% L0  = 1064;             %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% %  properties of CISe stack at 1064 nm
% n1 = 1;                 % air
% % n2 = 3.05 + 1j*0.15;    % Cu-poor CuInSe2 electrodeposited (Bhatia 2013)
% % n2 = 3.05 + 1j*0.30;    % Cu-rich CuInSe2 electrodeposited  (Bhatia 2013)
% n2 = 3.1 + 1j*0.20;     % Meadows modeling paper 2015
% n3 = 2.35 + 1j*4.43;    % Mo
% n4 = 1.5 + 1j*1e-7;     % SLG
% n  = [n1 n2 n3 n4];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 1200;              %  CuInSe2
% t3 = 400;
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% d34 = d23 + t3;
% D  = [d12 d23 d34];


% %%%%%%%%%%%%%%%%%%%%%%  Electrodeposited Cu2Se/In2Se3 on Mo on SLG example 
% %%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%
% %  Excitation parameters.
% L0  = 1064;             %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % properties of CISe stack at 1064 nm
% n1 = 1;                 % air
% n2 = 3.4 + 1j*1.2;    % compact Cu2-xSe   
% % n2 = 2.2 + 1j*0.6;      % Feathery Cu2-xSe with 50% volid (n=1+0j ) volume fraction. 
% n3 = 3.2 + 1j*0.3;      % compact In2Se3
% n4 = 2.35 + 1j*4.43;    % Mo
% n5 = 1.5 + 1j*1e-7;     % SLG
% n  = [n1 n2 n3 n4 n5];
% 
% %  Layer Thicknesses for each interior region (nm).
% % t2 = 300;              %  Cu2-xSe compact
% t2 = 600;              % Cu2-xSe feathery
% t3 = 800;               % In2Se3
% t4 = 400;               % Mo
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% d34 = d23 + t3;
% d45 = d34 + t4;
% D  = [d12 d23 d34 d45];



%%%%%%%%%%%%%%%%%%%%%%  Electrodeposited Cu/In/Se on Mo on SLG example 
%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%
%  Excitation parameters.
L0  = 1064;             %  Free-space wavelength of excitation (nm).
THi = 0*pi/180;         %  Incidence angle (rads).
TE  = 0;                %  Polarization.

% properties of CISe stack at 1064 nm
n1 = 1;                 % air
n2 = 2.8 + 1j*0.0;      % compact Se
n3 = 1.8 + 1j*8.4;      % compact In  
n4 = 0.38 + 1j*7.1;     % compact Cu 
n5 = 2.35 + 1j*4.43;    % Mo
n6 = 1.5 + 1j*1e-7;     % SLG
n  = [n1 n2 n3 n4 n5 n6];

%  Layer Thicknesses for each interior region (nm).
t2 = 500;              % Se
t3 = 350;              % In
t4 = 160;              % Cu 
t5 = 400;              % Mo

%  Locations for each planar boundary (nm).
d12 = 0;                %  Keep this as 0
d23 = d12 + t2;
d34 = d23 + t3;
d45 = d34 + t4;
d56 = d45 + t5;
D  = [d12 d23 d34 d45 d56];







% %%%%%%%%%%%%%%%%%%%%  CdTe at 248 nm example   %%%%%%%%%%%%%%%%%%%
% %  Excitation parameters.
% L0  = 248;              %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % CdTe at 808 nm
% n1 = 1;                 % air
% n2 = 3.99 + 1j*0.94;     % CdTe
% n3 = 2;                 % SLG
% n  = [n1 n2 n3];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 200;              %  CdTe
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% D  = [d12 d23];


% %%%%%%%%%%%%%%%%%%%  units calibration   %%%%%%%%%%%%%%%%%%%
% %  Excitation parameters.
% L0  = 500;              %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % CdTe at 808 nm
% n1 = 1;                 % air
% n2 = 3 + 1j*0.1;     % CdTe
% n3 = 1;                 % SLG
% n  = [n1 n2 n3];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 5000;              %  CdTe
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% D  = [d12 d23];


% %%%%%%%%%%%%%%%%%%%  CdTe at 355 nm example   %%%%%%%%%%%%%%%%%%%
% %  Excitation parameters.
% L0  = 355;              %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % CdTe at 808 nm
% n1 = 1;                 % air
% n2 = 4.17 + 1j*0.96;     % CdTe
% n3 = 2;                 % SLG
% n  = [n1 n2 n3];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 200;              %  CdTe
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% D  = [d12 d23];


% % %%%%%%%%%%%%%%%%%%%  CdTe superstrate metal back contact annealing at 1064 nm   %%%%%%%%%%%%%%%%%%%
% %  Excitation parameters.
% L0  = 1064;              %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% % 
% % CdTe at 1064 nm
% n1 = 1;                  % air
% n2 = 1.5;                % SLG
% n3 = 1.85 + 1j*8.5e-5;   % FTO
% n4 = 2.4 + 0.003;        % SnO2
% n5 = 2.3;                % CdS
% n6 = 2.7 + 1j*0.014;     % CdTe
% n7 = 2.35 + 1j*4.43;     % Mo
% n8 = 1;                  % air
% n  = [n1 n2 n3 n4 n5 n6 n7];
% 
% % n7 = 2.35 + 1j*4.43;     % Mo
% % n8 = 1;                  % air
% % n  = [n1 n2 n3 n4 n5 n6 n7 n8];
% % 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 7712;
% t3 = 360;
% t4 = 100;
% t5 = 80;
% t6 = 2100;
% % t7 = 500;
% % 
% %  Locations for each planar boundary (nm).
% d12 = 0        ;        %  Keep this as 0
% d23 = d12 + t2;
% d34 = d23 + t3;
% d45 = d34 + t4;
% d56 = d45 + t5;
% d67 = d56 + t6;
% % d78 = d67 + t7;
% 
% % D  = [d12 d23 d34 d45 d56 d67 d78];
% 
% D  = [d12 d23 d34 d45 d56 d67];





% %%%%%%%%%%%%%%%%%%%%  CdTe at 808 nm for Ferekides   %%%%%%%%%%%%%%%%%%%
% %  Excitation parameters.
% L0  = 808;              %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % CdTe at 808 nm
% n1 = 1;                 % air
% n2 = 3.0 + 1j*0.1664;   % CdTe
% n3 = 2;                 % SLG
% n  = [n1 n2 n3];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 400;              %  CdTe
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% D  = [d12 d23];



% %%%%%%%%%%%%%%%%%%%%%  AuZn on InP KrF  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Excitation parameters.
% L0  = 248;             %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% properties of CISe stack at 248 nm
% n1 = 1;                 % air
% n2 = 1.26 + 1j*1.3;     % AuZn
% n3 = 3.5 + 1j*2.1;      % InP
% n  = [n1 n2 n3];
% 
%  Layer Thicknesses for each interior region (nm).
% t2 = 60;              %  8 nm
% 
%  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% D  = [d12 d23];




% %%%%%%%%%%%%%%%%%%%%%%  InSb Wafer   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %  Excitation parameters.
% L0  = 1064;             %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % properties of CISe stack at 1064 nm
% n1 = 1;                 % air
% n2 = 4.2 + 1j*0.2;      % InSb
% n3 = 1;                 % air
% n  = [n1 n2 n3];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 5e5;              %  500 um
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% D  = [d12 d23];



% %%%%%%%%%%%%%%%%%%%%%%  532 nm Upsala CIGSe on Mo on SLG example
% %%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%
% %  Excitation parameters.
% L0  = 532;             %  Free-space wavelength of excitation (nm).
% THi = 0*pi/180;         %  Incidence angle (rads).
% TE  = 0;                %  Polarization.
% 
% % properties of CISe stack at 1064 nm
% n1 = 1;                 % air
% n2 = 1.8982 + 1j*0.146752;
% n3 = 1.8982 + 1j*0.146752;
% n4 = 2.14 + 0.09;
% n5 = 3.09 + 1j*0.18;    % CIGSe 
% change this for 532 not 1064    n6 = 2.35 + 1j*4.43;    % Mo
% n7 = 1.5 + 1j*1e-7;     % SLG
% n  = [n1 n2 n3 n4 n5 n6 n7];
% 
% %  Layer Thicknesses for each interior region (nm).
% t2 = 350;   %ZnO:Al             
% t3 = 90;    %i-ZnO
% t4 = 50;    %CdS
% t5 = 1700;  % CIGSe
% t6 = 350;   % Mo 
% 
% %  Locations for each planar boundary (nm).
% d12 = 0;                %  Keep this as 0
% d23 = d12 + t2;
% d34 = d23 + t3;
% d45 = d34 + t4;
% d56 = d45 + t5;
% d67 = d56 + t6;
% D  = [d12 d23 d34 d45 d56 d67];







%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% end sample definition  %%%%%%%%%%%%%%%%%%%%



% %%%%%%%%%%%%%%  Size of calculation domain.   %%%%%%%%%%%%%%%%%%%%%
% %  Specify axis boundaries (nm).
% xMin = -50;         % ignore x for normal-incidence problems
% xMax = 50;
% zMin = D(1) - D(end)/10;  % Z is propagation direction
% zMax = D(end) + D(end)/10;
% 
% %  Generate axes.
% Nx = 1;
% Nz = D(end) + 2*D(end)/10 + 1;   % this should end up giving 1 point per nm
% x  = linspace(xMin,xMax,Nx);
% z  = linspace(zMin,zMax,Nz);
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%  Size of calculation domain.   %%%%%%%%%%%%%%%%%%%%%
%  Specify axis boundaries (nm).
xMin = -50;         % ignore x for normal-incidence problems.  Only applies if you want to do a 2D problem with off-normal incidence
xMax = 50;
% zMin = D(1) - D(end)/10;  % Z is propagation direction
% zMax = D(end) + D(end)/10;
zMin = D(1);  % Z is propagation direction
zMax = D(end);

% zMin = -10;  % Z is propagation direction
% zMax = 2550;



%  Generate axes.
Nx = 1;
Nz = (zMax-zMin);
% Nz = 2541;   % number of points desired along z
x  = linspace(xMin,xMax,Nx);
z  = linspace(zMin,zMax,Nz);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%  Relative permittivity.
e_r = n.^2;

%  Relative permeability.
m_r = ones(size(e_r));

[R,T,AF,Sz,L] = laser_power_abs(L0,D,e_r,m_r,THi,TE,x,z);

%  Calculate divergence of the Poynting vector.
divS = central_difference(Sz,z(2) - z(1));

%  Positive divergence is technically radiated power while negative
%  divergence indicates absorbed power.  So let's add a negative sign so
%  that absorbed power becomes positive.
divS = -divS;

%  Calculate incident power density (assuming free-space).  
if TE
    Si = 1/(2*120*pi);
else
    Si = 120*pi/2;
end

%  Note the three ways to calculate total absorption factor.
AF1 = 1 - R - T;            %  Conservation of energy (this is method used by laser_power_abs)
AF2 = trapz(z,divS) / Si;   %  Integrate Poynting Divergence.  
AF3 = trapz(z,L) / Si;      %  Integrate volume absorption density.


diff_pwr_abs_analytical = L/Si;
diff_pwr_abs_numerical = divS/Si;



figure(1)
clf
plot(z,diff_pwr_abs_analytical,'k');
hold on
plot(z,diff_pwr_abs_numerical,'r');
xlabel('Position (nm)');
ylabel('Volumetric Absorption Density (normalized to incident power per area) (1/nm)');
legend('Analytical Value','Poynting Divergence');

%%%%%%% Print desired quantities to screen  %%%%%
R
T
AF
AF1;
AF2;
AF3;
Sz;
L;




%%%%%%%%%%% Save a 2 column file with columns z depth (nm) and differential power absorption (1/nm)  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
analytical_data = cat(2,z(:),diff_pwr_abs_analytical(:));        % using the analytical solution
numerical_data = cat(2,z(:),diff_pwr_abs_numerical(:));   % using the numerical derivative of Poynting
save_name = input('Type a filename root for saving.  Remember to put apostrophes around it name. (.dat will be added) =>');
if isempty(save_name)
    save_name = 'laser_power_abs_result';
end
dlmwrite(strcat(save_name,'_analytical.dat'),analytical_data,'\t');
dlmwrite(strcat(save_name,'_numerical.dat'),numerical_data,'\t');


